@@ -28,8 +28,6 @@
 #include "distribution.h"
 #include "proccess.h"
 
-#include "libglade-buddy.h"
-
 #include <stdio.h>
 #include <unistd.h>
 #include <string.h>
@@ -50,6 +48,10 @@
 
 #include <gconf/gconf-client.h>
 
+#ifdef HAVE_NETWORKMANAGER
+#include <libnm_glib.h>
+#endif
+
 #include <libsoup/soup.h>
 #include <libsoup/soup-xmlrpc-message.h>
 
@@ -66,6 +68,7 @@
 #define PROXY_PASSWORD	"/system/http_proxy/authentication_password"
 
 static GOptionData gopt_data;
+static int bug_count = 0;
 
 static const GOptionEntry options[] = {
 	{ "name",       '\0', 0, G_OPTION_ARG_STRING,   &gopt_data.name,        N_("Name of contact"),                    N_("NAME") },
@@ -80,7 +83,14 @@ static const GOptionEntry options[] = {
 	{ NULL }
 };
 
-void
+enum {
+	NETWORK_CONNECTED,
+	NETWORK_DISCONNECTED,
+	NETWORK_UNKNOWN
+};
+
+
+static void
 buddy_error (GtkWidget *parent, const char *msg, ...)
 {
 	GtkWidget *w;
@@ -106,36 +116,6 @@ buddy_error (GtkWidget *parent, const char *msg, ...)
 	g_free (s);
 }
 
-void
-on_gdb_stop_clicked (GtkWidget *button, gpointer data)
-{
-	GtkWidget *w;
-	/* FIXME */
-	w = gtk_message_dialog_new (NULL,
-				    0,
-				    GTK_MESSAGE_QUESTION,
-				    GTK_BUTTONS_YES_NO,
-				    _("gdb has not finished getting the "
-				      "debugging information.\n"
-				      "Kill the gdb process (the stack trace "
-				      "will be incomplete)?"));
-
-	gtk_dialog_set_default_response (GTK_DIALOG (w),
-					 GTK_RESPONSE_YES);
-
-	if (GTK_RESPONSE_YES == gtk_dialog_run (GTK_DIALOG (w))) {
-		gtk_widget_destroy (w);
-		//stop_gdb ();
-	}
-	gtk_widget_destroy (w);
-}
-
-GtkWidget *
-stock_pixmap_buddy (gchar *w, char *n, char *a, int b, int c)
-{
-	return gtk_image_new_from_stock (n, GTK_ICON_SIZE_SMALL_TOOLBAR);
-}
-
 static GnomeVersionInfo*
 get_gnome_version_info (void)
 {
@@ -229,18 +209,99 @@ link_callback (GtkLinkButton *button, gpointer user_data)
 	return;
 }
 
+static void
+save_to_file (const gchar *filename, const gchar *text)
+{
+	FILE          *bugfile;
+
+	if ((bugfile = fopen (filename, "w")) == NULL) {
+		g_warning ("Unable to save document %s\n", filename);
+		return;
+	}
+
+	fwrite (text, g_utf8_strlen (text, -1), 1, bugfile);
+
+	fclose (bugfile);
+}
+
+
+
+static void
+network_error (SoupMessage *msg, GladeXML *xml)
+{
+
+	GtkWidget *dialog;
+	int res;
+		
+	gtk_widget_hide (glade_xml_get_widget (xml, "progressbar"));
+	gtk_statusbar_push (GTK_STATUSBAR (glade_xml_get_widget (xml, "statusbar")),
+		    	    DONE, _("Done"));
+
+
+		
+	dialog = gtk_message_dialog_new_with_markup (NULL,
+						      GTK_DIALOG_MODAL,
+						      GTK_MESSAGE_QUESTION,
+						      GTK_BUTTONS_YES_NO,
+						      "<span weight=\"bold\">Network Connection Error</span>\n"
+						      "Maybe no Network Connection available.\n"
+						      "Do you want to store this report until "
+						      "a Network Connection is available?");
+	res = gtk_dialog_run (GTK_DIALOG (dialog));
+	gtk_widget_destroy (dialog);
+
+	if (res == GTK_RESPONSE_YES) {
+		gchar *dirname;
+		gchar *filename;
+		gchar *message_string;
+	       
+		dirname = g_strdup_printf ("%s/.gnome2/bug-buddy/pending_reports", g_get_home_dir ());
+		if (!g_file_test (dirname, G_FILE_TEST_IS_DIR)) {
+			g_mkdir_with_parents (dirname, 0755);
+		}
+
+		filename = g_strdup_printf ("%s/%ld", dirname, (long)time (NULL));
+					
+		message_string = soup_xmlrpc_message_to_string (SOUP_XMLRPC_MESSAGE (msg));
+
+		save_to_file (filename, message_string);
+
+		g_free (message_string);
+		g_free (dirname);
+		g_free (filename);
+	}
+		
+	gtk_main_quit();
+	return;
+}
 void
-bug_sent (SoupMessage *msg, GladeXML *xml)
+remove_pending_reports (void)
+{
+		GDir *dir;
+		char *dirname;
+		GError *error = NULL;
+
+		dirname = g_strdup_printf ("%s/.gnome2/bug-buddy/pending_reports", g_get_home_dir ());
+		dir = g_dir_open (dirname, 0, &error);
+		if (dir) {
+			const char *name = g_dir_read_name (dir);
+			while (name) {
+				char *path = g_strdup_printf ("%s/%s", dirname, name);
+				g_remove (path);
+				g_free (path);
+				name = g_dir_read_name (dir);
+			}
+			g_dir_close (dir);
+		}
+
+		g_remove (dirname);
+		g_free (dirname);
+}
+
+void
+all_sent (GladeXML *xml)
 {
-	char *text = NULL;
-	char *errmsg = NULL;
-	char *str = NULL;
-	char *ptr = NULL;
-	long bugid;
 	GtkWidget *close_button;
-	GtkWidget *main_vbox;
-	GtkWidget *urlbutton;
-	GError *err = NULL;
 
 	/* hide the progressbar */
 	gtk_widget_hide (glade_xml_get_widget (xml, "progressbar"));
@@ -251,15 +312,47 @@ bug_sent (SoupMessage *msg, GladeXML *xml)
 	gtk_widget_show (close_button);
 	gtk_statusbar_push (GTK_STATUSBAR (glade_xml_get_widget (xml, "statusbar")),
 			    DONE, _("Done"));
+}
+
+void
+previous_sent (SoupMessage *msg, GladeXML *xml)
+{
+	if (--bug_count == 0) {
+		all_sent (xml);
+	}
+}
+
+
+
+void
+bug_sent (SoupMessage *msg, GladeXML *xml)
+{
+	char *text = NULL;
+	char *errmsg = NULL;
+	char *str = NULL;
+	char *ptr = NULL;
+	long bugid;
+	GtkWidget *urlbutton;
+	GtkRequisition requisition;
+	GError *err = NULL;
+
+
+	if (SOUP_STATUS_IS_TRANSPORT_ERROR (msg->status_code)) {
+		network_error (msg, xml);
+	} else {
+		remove_pending_reports ();
+	}
 
-	/* we need a reference to the vbox containing the text so that we
-	 * can add a GtkLinkButton to the bug report */
-	main_vbox = glade_xml_get_widget (xml, "main-vbox");
-	
 	/* parse the XML-RPC response from bugzilla */
 	bugid = bugzilla_parse_response (msg, &err);
 
 	if (bugid > 0) {
+		GtkWidget *main_vbox;
+		
+		/* we need a reference to the vbox containing the text so that we
+	 	* can add a GtkLinkButton to the bug report */
+		main_vbox = glade_xml_get_widget (xml, "main-vbox");
+
 		text = g_strdup_printf ("http://bugzilla.gnome.org/show_bug.cgi?id=%d", bugid);
 
 		/* create a clickable link to the bug report */
@@ -349,6 +442,15 @@ bug_sent (SoupMessage *msg, GladeXML *xml)
 			g_error_free (err);
 		}
 	}
+	
+	if (--bug_count == 0) {
+		all_sent (xml);
+	}
+
+	gtk_widget_size_request (glade_xml_get_widget (xml, "main-window"), &requisition);
+	gtk_window_resize (GTK_WINDOW (glade_xml_get_widget (xml, "main-window")),
+			   requisition.width, requisition.height);
+	
 }
 
 
@@ -402,10 +504,6 @@ set_proxy (SoupSession *session)
 
 }
 
-
-
-
-
 static void
 send_report (BugzillaApplication *app, GnomeVersionInfo *gnome_version, GladeXML *xml)
 {
@@ -422,7 +520,8 @@ send_report (BugzillaApplication *app, GnomeVersionInfo *gnome_version, GladeXML
 	SoupXmlrpcResponse *response;
 	GError *err = NULL;
 
-
+	gtk_widget_hide (glade_xml_get_widget (xml, "pending-reports-check"));
+	
 	text_view = GTK_TEXT_VIEW (glade_xml_get_widget (xml, "gdb-text"));
 	buffer = gtk_text_view_get_buffer (text_view);
 	gtk_text_buffer_get_start_iter (buffer, &start);
@@ -435,10 +534,10 @@ send_report (BugzillaApplication *app, GnomeVersionInfo *gnome_version, GladeXML
 	gtk_text_buffer_get_end_iter (buffer, &end);
 	details_text = gtk_text_buffer_get_text (buffer, &start, &end, FALSE);
 
-	final_text = g_strdup_printf ("What were you doing when the app crashed?:\n%s\n\n\n%s", 
+	final_text = g_strdup_printf ("What were you doing when the application crashed?\n%s\n\n\n%s", 
 				      details_text != NULL ? details_text : "",
 				      gdb_text != NULL ? gdb_text : "<empty backtrace>");
-
+	
 	email = gtk_entry_get_text (GTK_ENTRY (glade_xml_get_widget (xml, "email-entry")));
 
 	message = bugzilla_create_report (app, gnome_version, email, final_text, xml, &err);
@@ -453,16 +552,53 @@ send_report (BugzillaApplication *app, GnomeVersionInfo *gnome_version, GladeXML
 		
 		buddy_error (NULL, text);
 		g_free (text);
+		g_free (gdb_text);
+		g_free (details_text);
+		g_free (final_text);
 		gtk_main_quit();
 		return;
 	}
 
-
 	session = soup_session_async_new ();
 	set_proxy (session);
 
-	soup_xmlrpc_message_persist (message);
 
+	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (glade_xml_get_widget (xml, "pending-reports-check")))) {
+		GDir *dir;
+		char *dirname;
+		GError *error = NULL;
+
+		dirname = g_strdup_printf ("%s/.gnome2/bug-buddy/pending_reports", g_get_home_dir ());
+		dir = g_dir_open (dirname, 0, &error);
+		if (dir) {
+			const char *name = g_dir_read_name (dir);
+			g_print ("There are pending reports!\n");
+			while (name != NULL) {
+				char *path;
+				char *contents;
+				
+				path = g_strdup_printf ("%s/%s", dirname, name);
+				if (g_file_get_contents (path, &contents, NULL, NULL)) {
+					SoupXmlrpcMessage *msg;
+					msg = soup_xmlrpc_message_new ("http://bugzilla.gnome.org/bugbuddy.cgi");
+					hacked_soup_xmlrpc_message_from_string (msg, contents);
+					bug_count++;
+					soup_xmlrpc_message_persist (msg);
+        				soup_session_queue_message (session, SOUP_MESSAGE (msg),
+               		                    			    previous_sent, xml);
+					g_free (contents);
+				}
+				g_free (path);
+				name = g_dir_read_name (dir);
+			}
+
+			g_dir_close (dir);
+		}
+	}
+
+	bug_count++;
+
+	soup_xmlrpc_message_persist (message);
         soup_session_queue_message (session, SOUP_MESSAGE (message),
                                     (SoupMessageCallbackFn)bug_sent, xml);
 
@@ -476,6 +612,7 @@ send_report (BugzillaApplication *app, GnomeVersionInfo *gnome_version, GladeXML
 }
 
 
+
 /* Check for *simple* email addresses of the form user@host, with checks
  * in characters used, and sanity checks on the form of host.
  */
@@ -553,6 +690,7 @@ on_send_clicked (GtkWidget *button, gpointer data)
 {
 	BugzillaApplication *app;
 	GnomeVersionInfo *gnome_version;
+	GtkRequisition requisition;
 	GladeXML *xml = (GladeXML*) data;
 
 	app = g_object_get_data (G_OBJECT (xml), "app");
@@ -565,24 +703,58 @@ on_send_clicked (GtkWidget *button, gpointer data)
 	gtk_widget_show (glade_xml_get_widget (xml, "progressbar"));
 	gtk_widget_hide (glade_xml_get_widget (xml, "final-box"));
 	gtk_widget_hide (glade_xml_get_widget (xml, "expander1"));
+
+	gtk_widget_size_request (glade_xml_get_widget (xml, "main-window"), &requisition);
+	gtk_window_resize (GTK_WINDOW (glade_xml_get_widget (xml, "main-window")),
+			   requisition.width, requisition.height);
+			   
 	send_report (app, gnome_version, xml);
 }
 
-void
-gdb_finished (gpointer data)
+static void
+gdb_insert_text (const gchar *stacktrace, GladeXML *xml)
+{
+	GtkTextView *text_view;
+	GtkTextIter end;
+	GtkTextBuffer *buffer;
+
+	text_view = GTK_TEXT_VIEW (glade_xml_get_widget (xml, "gdb-text"));
+	buffer = gtk_text_view_get_buffer (text_view);
+	gtk_text_buffer_get_end_iter (buffer, &end);
+
+	/* add the stacktrace to the GtkTextView */
+	gtk_text_buffer_insert (buffer, &end, stacktrace, strlen (stacktrace));
+
+}
+
+static void
+show_pending_checkbox_if_pending (GladeXML *xml)
+{
+	char *dirname;
+	GtkWidget *check;
+
+	dirname = g_strdup_printf ("%s/.gnome2/bug-buddy/pending_reports", g_get_home_dir ());
+	if (g_file_test (dirname, G_FILE_TEST_IS_DIR)) {
+		check = glade_xml_get_widget (xml, "pending-reports-check");
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check), TRUE);
+		gtk_widget_show (check);
+	}
+}
+
+static void
+known_app_finished (GladeXML *xml)
 {
-	BugzillaApplication *app;
-	GnomeVersionInfo *gnome_version;
-	GladeXML *xml = (GladeXML*) data;
 	GtkWidget *email_entry;
 	char *default_email;
-
+	
 	gtk_widget_show (glade_xml_get_widget (xml, "final-box"));
+	gtk_widget_show (glade_xml_get_widget (xml, "send-button"));
 	gtk_widget_hide (glade_xml_get_widget (xml, "progressbar"));
 	gtk_statusbar_pop (GTK_STATUSBAR (glade_xml_get_widget (xml, "statusbar")), DEBUGGING);
 
+
 	gtk_label_set_text (GTK_LABEL (glade_xml_get_widget (xml, "main-text")),
-			    _("The information about the crash has been succesfully collected. "
+			    _("The information about the crash has been successfully collected. "
 			      "Please provide some background about what you were doing when "
 			      "the application crashed.\n\n"
 			      "A valid email address is required.  This will allow developers to "
@@ -590,6 +762,9 @@ gdb_finished (gpointer data)
 			      "Sensitive data may be present in the debug information, so please "
 			      "review details below if you are concerned about transmitting "
 	                      "passwords or other sensitive data.\n"));
+
+	show_pending_checkbox_if_pending (xml);
+	
 	g_signal_connect (glade_xml_get_widget (xml, "send-button"), "clicked", 
 	                  G_CALLBACK (on_send_clicked), xml);
 
@@ -597,27 +772,136 @@ gdb_finished (gpointer data)
 	g_signal_connect (email_entry, "changed", G_CALLBACK (check_email), xml);
 
 	default_email = get_default_user_email ();
-	
+
 	if (default_email != NULL) {
 		gtk_entry_set_text (GTK_ENTRY (email_entry), default_email);
 		g_free (default_email);
 	}
 }
 
-
 static void
-close_callback (GtkWidget *widget, gpointer data)
+gdb_finished (const gchar *stacktrace, gpointer data)
 {
-	int pid;
-	GIOChannel *ioc;
-
 	GladeXML *xml = (GladeXML*) data;
 
-	ioc = g_object_get_data (G_OBJECT (xml), "ioc");
-	pid = (int)g_object_get_data (G_OBJECT (xml), "gdb-pid");
+	gdb_insert_text (stacktrace, xml);
+	known_app_finished (xml);
+}
 
-	if (ioc && pid) {
-		stop_gdb (ioc, pid);
+
+
+static void
+on_save_clicked (GtkWidget *button, gpointer user_data)
+{
+	GladeXML *xml = (GladeXML *)user_data;
+	GtkWidget *dialog;
+	gchar *desktop;
+	char *filename;
+	gboolean saved = FALSE;
+
+	dialog = gtk_file_chooser_dialog_new ("Save File",
+	                                      GTK_WINDOW (glade_xml_get_widget (xml, "main-window")),
+	                                      GTK_FILE_CHOOSER_ACTION_SAVE,
+	                                      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+	                                      GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
+	                                      NULL);
+
+	gtk_file_chooser_set_do_overwrite_confirmation (GTK_FILE_CHOOSER (dialog), TRUE);
+
+	desktop = g_strconcat (g_getenv ("HOME"), "/Desktop", NULL); 
+	gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog), desktop);
+
+	filename = g_strconcat (gopt_data.app_file, _("-bugreport.txt"), NULL);
+	gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (dialog), filename);
+	g_free (desktop);
+	g_free (filename);
+
+	if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT) {
+		char *filename;
+		GtkTextView   *text_view;
+		GtkTextBuffer *buffer;
+		GtkTextIter    start;
+		GtkTextIter    end;
+		gchar         *text;
+
+		text_view = GTK_TEXT_VIEW (glade_xml_get_widget (xml, "gdb-text"));
+		buffer = gtk_text_view_get_buffer (text_view);
+		gtk_text_buffer_get_start_iter (buffer, &start);
+		gtk_text_buffer_get_end_iter (buffer, &end);
+		text = gtk_text_buffer_get_text (buffer, &start, &end, FALSE);
+
+		filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
+		save_to_file (filename, text);
+		g_free (filename);
+		g_free (text);
+		saved = TRUE;
+	}
+
+	gtk_widget_destroy (dialog);
+	if (saved) {
+		gtk_main_quit ();
+	}
+}
+
+static void
+unknown_app_finished (GladeXML *xml)
+{
+	GtkWidget *button;
+
+	/* don't need user input, so hide these widgets */
+	gtk_widget_hide (glade_xml_get_widget (xml, "final-box"));
+	gtk_widget_hide (glade_xml_get_widget (xml, "progressbar"));
+	
+	/* make the send button into a save button :-) */
+	button = glade_xml_get_widget (xml, "send-button");
+	gtk_button_set_label (GTK_BUTTON (button), _("Save"));
+	g_signal_connect (GTK_BUTTON (button), "clicked", G_CALLBACK (on_save_clicked), xml);
+	gtk_widget_show (glade_xml_get_widget (xml, "send-button"));
+
+	/* expand the details about the stack trace */
+	gtk_expander_set_expanded (GTK_EXPANDER (glade_xml_get_widget (xml, "expander1")), TRUE);
+
+	gtk_statusbar_pop (GTK_STATUSBAR (glade_xml_get_widget (xml, "statusbar")), DEBUGGING);
+
+	gtk_label_set_text (GTK_LABEL (glade_xml_get_widget (xml, "main-text")),
+			    _("The information about the crash has been successfully collected.\n\n"
+	                      "The application that crashed is not known to bug-buddy, therefore the "
+	                      "bug report can not be sent to the GNOME Bugzilla.  Please save the "
+	                      "bug to a text file and report it to the appropriate bug tracker "
+	                      "for this application."));
+}
+
+static void
+gdb_finished_unknown_app (const gchar *stacktrace, gpointer data)
+{
+	GladeXML *xml = (GladeXML*) data;
+
+	gdb_insert_text (stacktrace, xml);
+	unknown_app_finished (xml);
+}
+
+
+
+
+static void
+close_callback (GtkWidget *widget, gpointer user_data)
+{
+	GladeXML *xml = (GladeXML *)user_data;
+	gpointer data;
+
+	g_return_if_fail (xml != NULL);
+
+	data = g_object_get_data (G_OBJECT (xml), "sourceid");
+
+	if (data != NULL) {
+		guint source_id = GPOINTER_TO_UINT (data);
+
+		/* removes the context from the main loop and kills any remaining
+	 	* gdb process */
+		if (source_id > 0) {
+			g_source_remove (source_id);
+			g_object_set_data (G_OBJECT (xml), "sourceid", GUINT_TO_POINTER (0));
+		}
 	}
 
 	gtk_main_quit ();
@@ -660,7 +944,6 @@ fill_gnome_info (BugzillaApplication *app, GnomeVersionInfo *gnome_version, Glad
 	gtk_text_buffer_insert (buffer, &end, version_info, strlen (version_info));
 
 	g_free (version_info);
-
 }
 
 
@@ -694,6 +977,29 @@ fill_proccess_info (pid_t pid, GladeXML *xml)
 
 }
 
+static void
+fill_include_file (char *filename, GladeXML *xml)
+{
+	GtkTextView *text_view;
+	GtkTextIter end;
+	GtkTextBuffer *buffer;
+	char *text;
+	GError *error = NULL;
+
+	if (g_file_get_contents (filename, &text, NULL, &error) != TRUE) {
+		buddy_error (NULL, error->message);
+		g_error_free (error);
+		return;
+	}
+	
+	text_view = GTK_TEXT_VIEW (glade_xml_get_widget (xml, "gdb-text"));
+	buffer = gtk_text_view_get_buffer (text_view);
+	gtk_text_buffer_get_end_iter (buffer, &end);
+	gtk_text_buffer_insert (buffer, &end, text, strlen (text));
+
+	g_free (text);
+
+}
 
 
 
@@ -712,6 +1018,8 @@ main (int argc, char *argv[])
 	GtkWidget *main_window, *statusbar;
 	GOptionContext *context;
 	GnomeProgram *program;
+	GError *err = NULL;
+	guint source_id;
 		
 	memset (&gopt_data,  0, sizeof (gopt_data));
 
@@ -731,12 +1039,8 @@ main (int argc, char *argv[])
 	                              GNOME_PARAM_APP_DATADIR, REAL_DATADIR,
 	                              NULL);
 
-	icon_info = gtk_icon_theme_lookup_icon (gtk_icon_theme_get_default (),
-						"bug-buddy", 48, 0);
-	if (icon_info) {
-		gnome_window_icon_set_default_from_file (gtk_icon_info_get_filename (icon_info));
-		gtk_icon_info_free (icon_info);
-        }
+	g_set_application_name (_("Bug-Buddy"));
+	gtk_window_set_default_icon_name ("bug-buddy");
 
 	s = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_APP_DATADIR,
 				       "bug-buddy/bug-buddy.glade", TRUE, NULL);
@@ -757,8 +1061,6 @@ main (int argc, char *argv[])
 
 	main_window = glade_xml_get_widget (xml, "main-window");
 	g_signal_connect (main_window, "delete-event", G_CALLBACK (delete_callback), xml);
-	g_signal_connect (glade_xml_get_widget (xml, "close-button"), "clicked", 
-	                  G_CALLBACK (close_callback), xml);
 	gtk_window_set_default_size (GTK_WINDOW (main_window), 400, 400);
 
 	gtk_widget_hide (glade_xml_get_widget (xml, "final-box"));
@@ -769,45 +1071,20 @@ main (int argc, char *argv[])
 	progress = g_timeout_add (100, update_progress_bar,
 				  glade_xml_get_widget (xml, "progressbar"));
 
-	if (gopt_data.app_file == NULL || gopt_data.pid == 0) {
-		buddy_error (NULL, _("Both the --pid and --appname arguments are required.\n"));	
+	if (gopt_data.app_file == NULL && gopt_data.package == NULL) {
+		buddy_error (NULL, _("Either --appname or --package arguments are required.\n"));	
 		g_object_unref (program);
 		return 0;
 	}	
 
-	apps = load_applications ();
-
-	app = g_hash_table_lookup (apps, gopt_data.app_file);
-	if (!app) {
-		char *text;
-
-		text = g_markup_printf_escaped (_("The crashed application (%s) does not provide\n"
-					        "a system where to send the debug information.\n"
-						"Please, check the about box of the application\n"
-						"to check how to report a bug."), gopt_data.app_file);
-		buddy_error (NULL, text);
-		g_free (text);
+	if (gopt_data.app_file && gopt_data.pid == 0 && gopt_data.include_file == NULL) {
+		buddy_error (NULL, _("Either --pid or --include arguments are required.\n"));	
 		g_object_unref (program);
 		return 0;
-	}
-	g_object_set_data (G_OBJECT (xml), "app", app);
-
-	s = g_markup_printf_escaped (_("The application %s has crashed.\n"
-					"We are collecting information about the crash to send it to the\n"
-					"developers in order to fix the problem."), app->name);	
-	gtk_label_set_text (GTK_LABEL (glade_xml_get_widget (xml, "main-text")), s);
-	g_free (s);
-
-	if (app->pixbuf)
-		gtk_image_set_from_pixbuf (GTK_IMAGE (glade_xml_get_widget (xml, "app-image")),
-					   app->pixbuf);
-
-
-	gtk_statusbar_push (GTK_STATUSBAR (statusbar),
-			    DEBUGGING, _("Collecting info from the crash..."));
-
+	}	
+	
+	/* get some information about the gnome version */
 	gnome_version = get_gnome_version_info ();
-
 	if (gnome_version == NULL) {
 		buddy_error (NULL, _("Bug Buddy was unable to retreive information regarding "
 		                     "the version of GNOME you are running.  This is most likely "
@@ -815,12 +1092,128 @@ main (int argc, char *argv[])
 		g_object_unref (program);
 		return 0;
 	}
-
-	fill_gnome_info (app, gnome_version, xml);
-	fill_proccess_info (gopt_data.pid, xml);
+	
 	g_object_set_data (G_OBJECT (xml), "gnome-version", gnome_version);
 
-	get_trace_from_gdb (gopt_data.app_file, gopt_data.pid, xml);
+	apps = load_applications ();
+	
+	/* If we have a binary file it is a crash */
+	if (gopt_data.app_file) {
+		app = g_hash_table_lookup (apps, gopt_data.app_file);
+
+		/* we handle an unknown application (no .desktop file) differently */
+		if (app != NULL) {
+			s = g_markup_printf_escaped (_("The application %s has crashed.\n"
+		                               		"We are collecting information about the crash to send it to the\n"
+                                               		"developers in order to fix the problem."), app->name);	
+			gtk_label_set_text (GTK_LABEL (glade_xml_get_widget (xml, "main-text")), s);
+			g_free (s);
+		
+			g_object_set_data (G_OBJECT (xml), "app", app);
+	
+			if (app->pixbuf)
+				gtk_image_set_from_pixbuf (GTK_IMAGE (glade_xml_get_widget (xml, "app-image")),
+			                           		app->pixbuf);
+			fill_gnome_info (app, gnome_version, xml);
+		}
+
+		gtk_statusbar_push (GTK_STATUSBAR (statusbar),
+			    	    DEBUGGING, _("Collecting info from the crash..."));
+
+		fill_proccess_info (gopt_data.pid, xml);
+
+		if (gopt_data.pid > 0) {
+			/* again, if this is an unknown application, we connect a different callback that
+	 		* will allow the user to save the trace rather than sending it to the GNOME bugzilla */
+			if (app == NULL) {
+				source_id = gdb_get_trace (gopt_data.app_file, gopt_data.pid, xml, 
+		                           		   gdb_finished_unknown_app, &err);
+			} else {
+				source_id = gdb_get_trace (gopt_data.app_file, gopt_data.pid, xml, gdb_finished, &err);
+			}
+
+			if (source_id == 0) {
+				buddy_error (NULL, _("Bug buddy encountered the following error when trying "
+		                     		     "to retrieve debugging information: %s\n"), err->message);
+				g_error_free (err);
+				g_object_unref (program);
+				return 0;
+			}
+
+			/* connect the close button callback so that we can remove the source from 
+	 		* the main loop (and kill gdb) if the user wants to quit before gdb is finished */
+			g_object_set_data (G_OBJECT (xml), "sourceid", GUINT_TO_POINTER (source_id));
+		} else {
+			if (app == NULL) {
+				unknown_app_finished (xml);
+			} else {
+				known_app_finished (xml);
+			}
+		}
+	} else {
+		/* No binary file, so this is a non-crashing bug. Look the application from the --package arg */
+		GtkWidget *email_entry;
+		char *default_email;
+		
+		app = g_hash_table_find (apps, bugzilla_search_for_package, gopt_data.package);
+		if (app == NULL) {
+			buddy_error (NULL, _("Bug Buddy doesn't know how to send a suggestion for the application %s.\n"),
+		                     	     gopt_data.package);
+			return 0;
+		}
+		s = g_markup_printf_escaped (_("Thank you for helping us improving our software.\n"
+				               "Please fill your suggestions/error information for %s application.\n"),
+                                              	app->name);
+		gtk_label_set_text (GTK_LABEL (glade_xml_get_widget (xml, "main-text")), s);
+		g_free (s);
+		
+		g_object_set_data (G_OBJECT (xml), "app", app);
+	
+		if (app->pixbuf)
+			gtk_image_set_from_pixbuf (GTK_IMAGE (glade_xml_get_widget (xml, "app-image")),
+	                           		   app->pixbuf);
+		fill_gnome_info (app, gnome_version, xml);
+	
+		gtk_widget_show (glade_xml_get_widget (xml, "final-box"));
+		gtk_widget_show (glade_xml_get_widget (xml, "send-button"));
+		gtk_widget_hide (glade_xml_get_widget (xml, "progressbar"));
+		gtk_statusbar_pop (GTK_STATUSBAR (glade_xml_get_widget (xml, "statusbar")), DEBUGGING);
+
+		s = g_markup_printf_escaped (_("Thank you for helping us improving our software.\n"
+					        "Please fill your suggestions/error information for %s application.\n\n"
+			      			"A valid email address is required.  This will allow developers to "
+			      			"contact you for more information if necessary.\n\n"),
+				             app->name);
+		gtk_label_set_text (GTK_LABEL (glade_xml_get_widget (xml, "main-text")), s);
+		g_free (s);
+
+		s = g_markup_printf_escaped (_("<span weight=\"bold\">Suggestion / Error description:</span>"));
+		gtk_label_set_markup (GTK_LABEL (glade_xml_get_widget (xml, "main-label")), s);
+		g_free (s);
+		
+		show_pending_checkbox_if_pending (xml);
+		g_signal_connect (glade_xml_get_widget (xml, "send-button"), "clicked", 
+	                  	  G_CALLBACK (on_send_clicked), xml);
+
+		email_entry = glade_xml_get_widget (xml, "email-entry");
+		g_signal_connect (email_entry, "changed", G_CALLBACK (check_email), xml);
+
+		default_email = get_default_user_email ();
+		
+		if (default_email != NULL) {
+			gtk_entry_set_text (GTK_ENTRY (email_entry), default_email);
+			g_free (default_email);
+		}
+		g_free (s);
+	}
+	
+	if (gopt_data.include_file != NULL) {
+		fill_include_file (gopt_data.include_file, xml);
+	}
+
+	
+	g_signal_connect (glade_xml_get_widget (xml, "close-button"), "clicked", 
+	                  G_CALLBACK (close_callback), xml);
 	
 	gtk_widget_show (main_window);
 
