@@ -22,7 +22,6 @@
 #include <config.h>
 
 #include "bug-buddy.h"
-#include "libglade-buddy.h"
 #include "distribution.h"
 
 #include <sys/stat.h>
@@ -133,7 +132,6 @@ add_bugzilla_application (GHashTable *hash,
 			s = s ? s+1 : programv[0];
 			if (g_hash_table_lookup (hash, s) == NULL) {
 				g_hash_table_insert (hash, g_strdup (s), app);
-				g_print ("DEBUG: Added %s\n", s);
 			} else {
 				g_free (app);
 				return;
@@ -151,7 +149,6 @@ add_bugzilla_application (GHashTable *hash,
 		for (i=0; programv[i]; i++) {
 			if (g_hash_table_lookup (hash, programv[i]) == NULL) {
 				g_hash_table_insert (hash, g_strdup (programv[i]), app);
-				g_print ("DEBUG: Added %s\n", programv[i]);
 			}
 		}
 		g_strfreev (programv);
@@ -317,8 +314,16 @@ get_all_applications (void)
 	GMenuTree          *tree;
 	GMenuTreeDirectory *root;
 	GSList             *retval;
+	const char         *menufile = BUDDY_DATADIR "/bug-buddy.menu";
 
-	tree = gmenu_tree_lookup ("applications.menu", GMENU_TREE_FLAGS_NONE);
+	if (g_file_test (menufile, G_FILE_TEST_IS_REGULAR)) {
+		/* use a custom menu file to scan desktop entry files so we aren't limited
+		 * to reporting bugs only present in the applications menu*/
+		tree = gmenu_tree_lookup (menufile, GMENU_TREE_FLAGS_INCLUDE_NODISPLAY);
+	} else {
+		/* fallback to using the applications menu */
+		tree = gmenu_tree_lookup ("applications.menu", GMENU_TREE_FLAGS_INCLUDE_NODISPLAY);
+	}
 
 	root = gmenu_tree_get_root_directory (tree);
 
@@ -407,6 +412,18 @@ load_applications (void)
 	return program_to_application;
 }
 
+gboolean
+bugzilla_search_for_package (gpointer key, gpointer value, const char *package)
+{
+	BugzillaApplication *app = (BugzillaApplication*) value;
+
+	if (!strcmp (app->product, package))
+		return TRUE;
+	
+	return FALSE;
+}
+
+
 int
 bugzilla_parse_response (SoupMessage *msg, GError **err)
 {
@@ -430,8 +447,6 @@ bugzilla_parse_response (SoupMessage *msg, GError **err)
 		return -1;
 	}
 
-	g_print ("DEBUG %s\n", soup_xmlrpc_response_to_string (response));
-
 	/* check to see if the XMLRPC response was a <fault> */
 	if (soup_xmlrpc_response_is_fault (response)) {
 		SoupXmlrpcValue *faultCode = NULL;
